<?php

namespace app\controllers;

use Yii;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\web\Response;
use yii\filters\VerbFilter;
use app\models\LoginForm;
use app\models\ContactForm;
use app\models\PaiementForm;
use app\models\Paiement;
use yii\helpers\VarDumper;
use yii\helpers\Url;
use app\components\PaiementRetrieverJob;

class SiteController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'only' => ['logout'],
                'rules' => [
                    [
                        'actions' => ['logout'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'logout' => ['post'],
                    'index' => ['post', 'get'],
                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    /**
     * Displays payment page.
     *
     * @return string
     */
    public function actionIndex()
    {
      error_log("action index");
      $default=['shop_name' => SHOP_NAME,
               'logo_url' => LOGO_URL,
               'amount' => AMOUNT,
               'description' => PURCHASE_DESCRIPTION,
               'number_orange' => NUMBER_ORANGE,
               'number_telmob' => NUMBER_TELMOB,
               'return_url' => RETURN_URL,
               'accessKey' => API_ACCESS_KEY,
               'immutableRowIds' => IMMUTABLE_ROW_IDS,
             ];
        $decOrig=null; $sumOrig=null; $idOrig=null; $result=[]; $privKey=null; $accessKey=null; 
        if(Yii::$app->request->getQueryParams('enc') != null) {
            Yii::debug("data was set via GET");
            $decOrig=Yii::$app->request->getQueryParam('enc');
            $sumOrig=Yii::$app->request->getQueryParam('sum');
            $idOrig=Yii::$app->request->getQueryParam('id');
        }else if(Yii::$app->request->getBodyParams('enc') != null) {
            Yii::debug("data was set by POST");
            $decOrig=Yii::$app->request->getBodyParam('enc');
            $sumOrig=Yii::$app->request->getBodyParam('sum');
            $idOrig=Yii::$app->request->getBodyParam('id');
        }
        $dec=base64_decode($decOrig);
        $sum=base64_decode($sumOrig);
        $id=base64_decode($idOrig);
        
        if($dec != null  && $sum != null) {
            Yii::debug('$dec = ' . VarDumper::dumpAsString($dec),__METHOD__);
            Yii::debug('$sum = '. $sum,__METHOD__);
            if($id != null) {
              $privKey=Yii::$app->remoteDb->createCommand('select privateKey from profile where id=:id')->bindValue(':id', $id)->queryScalar();
              $accessKey=Yii::$app->remoteDb->createCommand('select access_token from user where id=:id')->bindValue(':id', $id)->queryScalar();
              Yii::debug('using private key from db user id='.$id);
            } else if(defined('PRIVATE_KEY')) {
                Yii::debug('using private key from config');
                $privKey=PRIVATE_KEY;
                $accessKey=$default['accessKey'];
            }
            if($privKey != null) {
                while ($msg = openssl_error_string()) {};
                $tmp=$sum;
                $rv=openssl_private_decrypt( $tmp, $sum, $privKey);
                Yii::debug('openssl_private_decrypt returned '.VarDumper::dumpAsString($rv));
                $str="";
                while ($msg = openssl_error_string())
                    $str.=$msg . "\n";
                if($str) Yii::debug($str);
            }
            if(hash('sha256',$dec)==$sum) {
                parse_str($dec,$result);
                Yii::debug('$sum check OK: $result = '.VarDumper::dumpAsString($result));
/*                if(isset($result['amount'])) {
                    $amount=$result['amount'];
                    Yii::debug('$amount = ' . VarDumper::dumpAsString($amount),__METHOD__);
                }
                if(isset($result['return_url'])) {
                    $return_url=$result['return_url'];
                    Yii::debug('$return_url = ' . VarDumper::dumpAsString($return_url),__METHOD__);
                }
*/            }
        }
        // !isset($return_url) && defined('RETURN_URL') && $return_url=RETURN_URL;
        // !isset($amount) && defined('AMOUNT') && $amount=AMOUNT;

        // if(!(isset($amount) && isset($return_url))) {
        //     Yii::warning('La variable ' . (isset($amount)? '$return_url' : '$amount' )
        //                 .' doit être définie,'
        //                 .' via la QUERY_STRING ou via le fichier config/constants.php.'
        //                 ,__METHOD__);
        //     $this->redirect(HOME_URL);
        // } else {
            $consolidatedParams=array_merge($default, $result);
            $consolidatedParams['return_url'] = urldecode($consolidatedParams['return_url']);
            // $immutableRowIds=(isset($result['immutableRowIds']) ? $result['immutableRowIds'] : $default['immutableRowIds']);
            Yii::trace('$accessKey = ' . $accessKey);
            $paiementForm=new PaiementForm();
            if ($paiementForm->load(Yii::$app->request->post()) && $paiementForm->validate()
                    && $paiement=$paiementForm->fetch(
                                    $accessKey,
                                    SiteController::getMontantsFromAmountsArray($consolidatedParams['amount']),
                                    $consolidatedParams['immutableRowIds'])
                                    ) {
                //Si on sauvegarde l'enregistrement, il va générer un nouvel id, sinon, on reste
                //avec l'id en provenance du serveur.
                Yii::trace('old id = ' . $paiement->id);
                if(defined('SAVE_LOCAL_RECORD') && SAVE_LOCAL_RECORD) {
                    $paiement->id=null;
                    $paiement->save();
                    $paiement->refresh();
                    Yii::trace('new id = ' . $paiement->id);
                }
                $return_url = $consolidatedParams['return_url'];
                $return_url = str_replace('%ID%', $paiement->id, $return_url);
                $return_url = str_replace('%TRANSACTION_ID%', $paiement->transaction_id, $return_url);
                $return_url = str_replace('%AMOUNT%', $paiement->amount, $return_url);
                Yii::debug('$return_url = ' . VarDumper::dumpAsString($return_url));
                $this->redirect($return_url);
            }
            //si les paramètres ont été passé par POST, il faut les rendre disponible dans la vue pour renvoi à la prochaine exécution
            if(isset(Yii::$app->request->bodyParams['enc']))
                $params=array_merge(['paiementForm' => $paiementForm], $consolidatedParams, ['enc' => $decOrig, 'sum' => $sumOrig]);
            else
                $params=array_merge(['paiementForm' => $paiementForm], $consolidatedParams);
            Yii::debug('$params = '.VarDumper::dumpAsString($params));
            return $this->render('index', $params);
       // }
    }

    /**
    * Crée un tableau des montants (et rien que les montants) à partir du tableau amount
    * (contenant montants, libellés et durée PHP) reçu du site web marchand ou paramétré
    * dans les constantes
    * @param array $amountsArray le tableau amount complet
    * @return array le tableau des montants
    */
    public static function getMontantsFromAmountsArray($amountsArray)
    {
        //maintenant, tous les montants devrait être toujours sous forme d'array
        if(is_array($amountsArray)) {
            foreach ($amountsArray as $key => $value) {
                $montants[]=$value['montant'];
            }
        } else {
            $montants[]=$amountsArray;
        }
        return $montants;
    }

    /**
     * incoming action: receives paiement id from MPay server and enqueues job to retrieve data
     * @param int $recordId Identifiant de l'enregistrement à rapatrier
     * @return Response|string
     */
    public function  actionIncoming($recordId)
    {
        $job=new PaiementRetrieverJob(['remoteId' => $recordId]);
        Yii::$app->queue->push($job);
        Yii::$app->response->statusCode = 200;
        return;
    }

}
